#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "screen.h"
#include "demo.h"

static int init(void);
static void destroy(void);
static void start(long trans_time);
static void stop(long trans_time);
static void draw(void);
static float smoothstep(float a, float b, float x);

static struct screen scr = {
	"console",
	init,
	destroy,
	start,
	stop,
	draw
};

static void *saved_fb;
static int fbsize;
static long trans_start = -1, trans_dur;

#define NBLOCKS		32
#define MAX_DELAY	0.5
static float delay[NBLOCKS];
static int blksz;

struct screen *console_screen(void)
{
	return &scr;
}

static int init(void)
{
	int i;
	fbsize = fb_width * fb_height * fb_depth / 8;

	if(!(saved_fb = malloc(fbsize))) {
		perror("failed to allocate console framebuffer");
		return -1;
	}
	memcpy(saved_fb, fb_pixels, fbsize);

	blksz = fb_height / NBLOCKS;

	for(i=0; i<NBLOCKS; i++) {
		delay[i] = (float)rand() / (float)RAND_MAX * MAX_DELAY;
	}

	return 0;
}

static void destroy(void)
{
	memcpy(fb_pixels, saved_fb, fbsize);
	free(saved_fb);
}

static void start(long trans_time)
{
}

static void stop(long trans_time)
{
	trans_start = time_msec;
	trans_dur = trans_time;
}

static void draw(void)
{
	int i, pixsz = fb_depth / 8;
	long elapsed, offs, rem;
	unsigned char *dptr = fb_pixels;
	unsigned char *sptr = saved_fb;

	if(trans_start < 0) {
		return;
	}

	elapsed = time_msec - trans_start;

	for(i=0; i<fb_height; i++) {
		int bidx = i / blksz;
		float t = (float)elapsed / (float)trans_dur;
		offs = fb_width * smoothstep(0, 1.0 - delay[bidx], t - delay[bidx]);
		if(offs < 0) offs = 0;
		rem = fb_width - offs;

		if(offs > 0) {
			memset(dptr, 0, offs * pixsz);
		}
		if(rem > 0) {
			memcpy(dptr + offs * pixsz, sptr, rem * pixsz);
		}

		dptr += fb_width * pixsz;
		sptr += fb_width * pixsz;
	}
}

static float smoothstep(float a, float b, float x)
{
	if(x < a) return 0.0;
	if(x >= b) return 1.0;

	x = (x - a) / (b - a);
	return x * x * (3.0 - 2.0 * x);
}
