#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "fbgfx.h"
#include "fbevents.h"
#include "timer.h"
#include "screen.h"
#include "screen.h"
#include "demo.h"

static void keyboard(int key, int pressed, void *cls);
static void mouse(int bn, int pressed, int x, int y, void *cls);
static void motion(int x, int y, void *cls);

static long start_msec, num_frames;

static int quit;


int main(void)
{
	int i, trybpp[] = {32, 24, 16, 0};

	fbgfx_save_video_mode();
	fbgfx_get_video_mode(&fb_width, &fb_height, &fb_depth);

	for(i=0; trybpp[i]; i++) {
		if(!(fb_pixels = fbgfx_set_video_mode(fb_width, fb_height, trybpp[i]))) {
			continue;
		}
		fbgfx_get_video_mode(&fb_width, &fb_height, &fb_depth);
		if(fb_depth == trybpp[i]) {
			break;
		}
		fprintf(stderr, "failed to set color fb_depth: %dbpp\n", trybpp[i]);
	}
	if(trybpp[i] == 0) {
		fprintf(stderr, "no usable color fb_depths found\n");
		goto end;
	}

	if(fbev_init() == -1) {
		goto end;
	}
	fbev_keyboard(keyboard, 0);
	fbev_mbutton(mouse, 0);
	fbev_mmotion(motion, 0);

	if(scr_init() == -1) {
		goto end;
	}
	scr_change(scr_lookup("console"), 0);
	scr_change(scr_lookup("tunnel"), 4000);

	start_msec = get_time_msec();
	for(;;) {
		fbev_update();
		if(quit) break;

		time_msec = get_time_msec() - start_msec;

		scr_update();
		scr_draw();
		++num_frames;
	}

	time_msec = get_time_msec() - start_msec;
end:
	scr_shutdown();
	fbev_shutdown();
	fbgfx_restore_video_mode();
	if(num_frames && time_msec) {
		printf("\ravg framerate: %.1f\n", (float)num_frames / ((float)time_msec / 1000.0));
	}
	return 0;
}

static void keyboard(int key, int pressed, void *cls)
{
	if(!pressed) return;

	switch(key) {
	case 27:
	case 'q':
	case 'Q':
		quit = 1;
		break;
	}
}

static void mouse(int bn, int pressed, int x, int y, void *cls)
{
}

static void motion(int x, int y, void *cls)
{
}
